Sparse-checkout from NVIDIA/cccl main branch to complete cccl_upstream: Added: - python/cuda_cccl/ (226 files) — Python bindings for device-level algorithms Critical for muh toolchain: cuda.compute.reduce_into, scan, radix_sort, etc. Includes 204 .py files with full test coverage for all 27 algorithms - ci/ (163 files) — Build/test infrastructure build_cub.sh, test_cub.sh, build_and_test_targets.sh, matrix.yaml Directly maps to our [INFRA-CI] and [INFRA-BUILD] items - .agent/skills/ (7 files) — NVIDIA's own agent skills for CCCL cccl-style/SKILL.md, cccl-test/SKILL.md, sass-diff/SKILL.md - docs/ (491 files) — Official CCCL documentation CI references, CMake guides, Python compute docs, libcudacxx PTX docs - test/ (12 files) — Top-level integration tests (cuda_smoke, stdpar) - Root configs: .clang-format, .clang-tidy, CONTRIBUTING.md, pyproject.toml - CLAUDE.md symlink → AGENTS.md (NVIDIA's standard) cccl_upstream now mirrors full NVIDIA/cccl structure: Before: 42M (cub + thrust + libcudacxx + cudax + c + examples + benchmarks) After: 53M (+python +ci +docs +.agent +test +configs) This completes the CCCL base needed for: - [muh-bench] items: ci/util/build_and_test_targets.sh for targeted builds - [CCCL-verify] items: python/cuda_cccl/tests/ as reference implementations - [CCCL-test] items: ci/test_cub.sh, ci/test_thrust.sh - Agent workflow: .agent/skills/ for consistent style and test patterns
69 lines
1.6 KiB
ReStructuredText
69 lines
1.6 KiB
ReStructuredText
.. _libcudacxx-extended-api-math-sincos:
|
|
|
|
``cuda::sincos``
|
|
====================================
|
|
|
|
Defined in the ``<cuda/cmath>`` header.
|
|
|
|
.. code:: cuda
|
|
|
|
namespace cuda {
|
|
|
|
template <class T>
|
|
struct sincos_result
|
|
{
|
|
T sin;
|
|
T cos;
|
|
};
|
|
|
|
template </*floating-point-type*/ T>
|
|
[[nodiscard]] __host__ __device__
|
|
sincos_result<T> sincos(T value) noexcept; // (1)
|
|
|
|
template <class Integral>
|
|
[[nodiscard]] __host__ __device__
|
|
sincos_result<double> sincos(Integral value) noexcept; // (2)
|
|
|
|
} // namespace cuda
|
|
|
|
Computes :math:`\sin value` and :math:`\cos value` at the same time using more efficient algorithms than if operations were computed separately.
|
|
|
|
**Parameters**
|
|
|
|
- ``value``: The input value.
|
|
|
|
**Return value**
|
|
|
|
- ``cuda::sincos_result`` object with both values set to ``NaN`` if the input value is :math:`\pm\infty` or ``NaN`` and to results of :math:`\sin value` and :math:`\cos value` otherwise. (1)
|
|
- if ``T`` is an integral type, the input value is treated as ``double``. (2)
|
|
|
|
**Constraints**
|
|
|
|
- ``T`` is an arithmetic type.
|
|
|
|
**Performance considerations**
|
|
|
|
- If available, the functionality is implemented by compiler builtins, otherwise fallbacks to ``cuda::std::sin(value)`` and ``cuda::std::cos(value)``.
|
|
|
|
Example
|
|
-------
|
|
|
|
.. code:: cuda
|
|
|
|
#include <cuda/cmath>
|
|
#include <cuda/std/cassert>
|
|
|
|
__global__ void sincos_kernel() {
|
|
auto [sin_pi, cos_pi] = cuda::sincos(0.f);
|
|
assert(sin_pi == 0.f);
|
|
assert(cos_pi == 1.f);
|
|
}
|
|
|
|
int main() {
|
|
sincos_kernel<<<1, 1>>>();
|
|
cudaDeviceSynchronize();
|
|
return 0;
|
|
}
|
|
|
|
`See it on Godbolt 🔗 <https://godbolt.org/z/99PP9s1z6>`__
|